////////////////////////////////////////////////////////////////////////////
//
// COPYRIGHT ANSYS. ALL RIGHTS RESERVED.
//
// $LastChangedDate$
// $LastChangedRevision$
// $LastChangedBy$
//
// Author: ltomaso
//
// Decription : header declaring the CppOptimizer classes and routines
//
////////////////////////////////////////////////////////////////////////////

// The following ifdef block is the standard way of creating macros which simplify the process of exporting 
// from a DLL. All files within this DLL are compiled with the CPPOPTIMIZEREXAMPLE_EXPORTS
// symbol defined on the command line. This symbol should not be defined in any project
// that uses this DLL. This way, any other project with source files that include this file sees 
// CPPOPTIMIZEREXAMPLE_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef WIN32
#ifdef CPPOPTIMIZEREXAMPLE_EXPORTS
#define CPPOPTIMIZEREXAMPLE_API __declspec(dllexport)
#else
#define CPPOPTIMIZEREXAMPLE_API __declspec(dllimport)
#endif
#else
#define CPPOPTIMIZEREXAMPLE_API 
#endif

#include <vector>
#include <map>
#include <cmath>

#ifdef __cplusplus
extern "C" 
{
#endif


    CPPOPTIMIZEREXAMPLE_API void AddDoubleVariable(int iId, double dLowerBound, double dUpperBound, double dInitialValue);
    CPPOPTIMIZEREXAMPLE_API void AddObjective(int iId, int iObjectiveType, double dTargetValue, double dImportance);
    CPPOPTIMIZEREXAMPLE_API void AddConstraint(int iId, int iConstraintType, double dConstraintFirstValue, double dConstraintSecondValue, double dImportance);
    CPPOPTIMIZEREXAMPLE_API void PutNumberOfSamples(int iNumSamples); 
    CPPOPTIMIZEREXAMPLE_API void PutMaximumNumberOfCandidates(int iMaxNumCandidates); 
    CPPOPTIMIZEREXAMPLE_API bool CanRun(void); 
    CPPOPTIMIZEREXAMPLE_API int RunOptimization(); 
    CPPOPTIMIZEREXAMPLE_API void GetInputValues(int iSubmittedPointId, double *dValues); 
    CPPOPTIMIZEREXAMPLE_API void AddOutputValues(int iSubmittedPointId, bool success, double *dValues, int iNumValues); 
    CPPOPTIMIZEREXAMPLE_API int GetNumberOfGeneratedCanidatePoints(void); 
    CPPOPTIMIZEREXAMPLE_API void GetCandidatePointValues(int iCandidatePointId, double *dValues);
    CPPOPTIMIZEREXAMPLE_API void GetCostFunctionValues(double *cost, bool *feasible); 
    CPPOPTIMIZEREXAMPLE_API wchar_t * GetStatus();
    CPPOPTIMIZEREXAMPLE_API void Clear();

    // enumeration for the variable type
    typedef enum E_VariableType
    {
        eDouble = 0,
        eDoubleList = 1,
        eIntegerList = 2
    } E_VariableType;

    // enumeration for the objective type
    typedef enum E_ObjectiveType
    {
        eMinimize = 0,
        eMaximize = 1,
        eSeekTarget = 2
    } E_ObjectiveType;

    // enumeration for the constraint type
    typedef enum E_ConstraintType
    {
        eLessThan = 0,
        eGreaterThan = 1,
        eEqualTo = 2,
        eInsideBounds = 3
    } E_ConstraintType;

    // class containing the definition of input parameters
    class CPPOPTIMIZEREXAMPLE_API MyParameter
    {
    public:
        MyParameter(){}
        ~MyParameter()
        {
            _values.clear();
        }

    public:
        int _id;// unique id
        E_VariableType _type;// type of variable
        double _initialvalue;// intial value (used when the optimizer starts from an initial point)
        std::vector<double> _values;// corresponds to the lower and the upper bounds for a double variable; otherwise, corresponds to list of values

    };

    // class containing the definition of the objective
    class CPPOPTIMIZEREXAMPLE_API MyObjective
    {
    public:
        MyObjective(){}
        ~MyObjective(){}

    public:
        int _id;// parameter id
        E_ObjectiveType _type;// type of objective
        double _targetvalue;// target value (only used when _type == eSeekTarget)
        double _importance;// importance of the objective (can be used to build the cost function to convert a multiple-objective optimization to  a single objective optimization)
    };

    // class containing the definition of the constraint
    class CPPOPTIMIZEREXAMPLE_API MyConstraint
    {
    public:
        MyConstraint(){}
        ~MyConstraint(){}

    public:
        int _id;
        E_ConstraintType _type;// type of constraint
        double _constraintFirstValue;// corresponds to the lower bound if _type==eLessThan or eInsideBounds, or the upper bound if _type==eGreaterThan, or the target value if _type==eEqualTo
        double _constraintSecondValue;// corresponds to the upper bound if _type==eInsideBounds (only used in this case)
        double _importance;// importance of the constraint can be used to weight the constraint in the building of the cost function
    };

    // class containing the definition of the point with input and output parameter values, its status, and its cost function value...
    class CPPOPTIMIZEREXAMPLE_API MyPoint
    {
    public:
        MyPoint()
        {
            _submitted = false;
            _uptodate = false;
            _success = false;
            _feasible = false;
        }
        ~MyPoint()
        {
            _inputs.clear();
            _outputs.clear();
        }

    public:
        std::vector<double> _inputs;// input parameter values
        std::vector<double> _outputs;// output parameter values
        bool _submitted;
        bool _uptodate;// true if the point has been evaluated
        bool _success;// true if the evaluation has been done successfully without any error
        double _costfunction;// cost function value (based on constraint violation and objective values)
        bool _feasible;// true if there is no constraint violation
    };

    class CPPOPTIMIZEREXAMPLE_API MyCppOptimizer
    {
    public:
        MyCppOptimizer()
        {
            _iNumInputs = 0;
            _iNumOutputs = 0;
            _iNumObjectives = 0;
            _iNumConstraints = 0;
            _iMaxNumPoints = 0;
            _iMaxNumCandidates = 0;
        }
        MyCppOptimizer(MyCppOptimizer &);
        ~MyCppOptimizer()
        {
            Clear();
        }


        void Init();
        void Clear();

    public:

        std::vector<MyParameter> _params;// list of all parameters (inputs + outputs)
        std::vector<MyParameter> _inputs;// list of input parameters
        std::vector<MyObjective> _objectives;// list of objectives
        std::vector<MyConstraint> _constraints;// list of constraints
        std::vector<MyPoint> _pointsToSubmit;// list of submitted points
        std::vector<MyPoint> _allpoints;// list of all evaluated points
        std::vector<MyPoint> _candidates;// list of candidate points
        int _iNumInputs;// number of input parameters
        int _iNumOutputs;// number of output parameters
        int _iNumObjectives;// number of objectives
        int _iNumConstraints;// number of constraints
        int _iMaxNumCandidates;// maximum number of candidates
        int _iMaxNumPoints;// maximum number of points to generate
        double _bestcostvalue;
        bool _feasiblecandidate;
    };

#ifdef __cplusplus
}
#endif
